# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

import time

class FsUpdate(object):
  """
  This models the FsUpdate Avro record and gives it a bit more structure.
  """
  def __init__(self, mountpoint, partition, timestamp=None):
    if timestamp is None:
      timestamp = time.time()
    self._timestamp = timestamp
    self._mount = mountpoint
    self._partition = partition
    self._metrics = [ ]

  def add_metric(self, metricid, value):
    self._metrics.append((metricid, value))
    return self

  def __str__(self):
    sio = StringIO()
    sio.write(">> %s:" %
              time.strftime("%x %X", time.localtime(self._timestamp)))
    sio.write(">>  %s (%s)" % (self._mount, self._partition))
    for metricid, val in self._metrics:
      sio.write("\n   %-10d : %s" % (metricid, val))
    return sio.getvalue()

  def to_avro_dict(self):
    """Return a Python dictionary that maps to the Avro struct"""
    return dict(mountpoint=self._mount,
                partition=self._partition,
                metrics=[ dict(id=k, value=v) for (k, v) in self._metrics ])
