-- Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.
--
-- Version 1.0 of the Oracle schema file
-- The filename starts with vesion 25 to keep it in sync with
-- mysql and postgres releases

create table SCHEMA_VERSION (
    VERSION number(4, 0) constraint NOT_NULL_VERSION not null,
    OLD_VERSION number(4, 0) constraint NOT_NULL_OLD_VERSION not null
);

-- Update this value when the schema changes, also during migration
insert into SCHEMA_VERSION values (26, 0);

create table AUDITS (
    AUDIT_ID number constraint NOT_NULL_AUDIT_ID not null,
    ACTING_USER_ID number,
    AUDIT_TYPE varchar2(255) default 'UNKNOWN' constraint NOT_NULL_AUDIT_TYPE not null,
    SERVICE_ID number,
    ROLE_ID number,
    CONFIG_REVISION_ID number,
    COMMAND_ID number,
    USER_ID number,
    HOST_ID number,
    CREATED_INSTANT number,
    MESSAGE clob,
    constraint PK_AUDITS primary key (AUDIT_ID)
);

create table CLUSTERS (
    CLUSTER_ID number constraint NOT_NULL_CLUSTER_ID not null,
    NAME varchar2(255),
    constraints PK_CLUSTERS primary key (CLUSTER_ID)
);

create table COMMANDS (
    COMMAND_ID number constraint NOT_NULL_CMD_ID not null,
    NAME varchar2(255) constraint NOT_NULL_CMD_NAME not null,
    STATE varchar2(255),
    START_INSTANT number,
    END_INSTANT number,
    ACTIVE number,
    RESULT_MESSAGE clob,
    RESULT_DATA blob,
    RESULT_DATA_MIME_TYPE varchar2(255),
    RESULT_DATA_FILENAME varchar2(255),
    RESULT_DATA_PATH varchar2(255),
    RESULT_DATA_REAPED number(1,0) default 0,
    SUCCESS number(1,0),
    INTERNAL_STATE blob,
    SERVICE_ID number,
    ROLE_ID number,
    HOST_ID number,
    PARENT_ID number,
    constraint PK_COMMANDS primary key (COMMAND_ID)
);

create table CONFIGS (
    CONFIG_ID number constraint NOT_NULL_CONFIG_ID not null,
    ROLE_ID number,
    ROLE_TYPE varchar2(255),
    ATTR varchar2(255) constraint NOT_NULL_ATTR not null,
    VALUE clob constraint NOT_NULL_VALUE not null,
    REVISION_ID number constraint NOT_NULL_CONFIS_REV_ID not null,
    SERVICE_ID number constraint NOT_NULL_CONFIGS_SRV_ID not null,
    constraint PK_CONFIGS primary key (CONFIG_ID)
);

create table CONFIG_REVISIONS (
    REVISION_ID number constraint NOT_NULL_REVISION_ID not null,
    USER_ID number,
    SERVICE_ID number constraint NOT_NULL_CR_SRV_ID not null,
    CREATED_INSTANT number constraint NOT_NULL_CR_CREATED_INSTANT not null,
    MESSAGE clob constraint NOT_NULL_MESSAGE not null,
    VISIBLE number default 0 constraint NOT_NULL_VISIBLE not null,
    constraint PK_CONFIG_REVISIONS primary key (REVISION_ID)
);

create table CREDENTIALS (
    CREDENTIAL_ID number constraint NOT_NULL_CREDENTIAL_ID not null,
    PRINCIPAL varchar2(255) constraint NOT_NULL_PRINCIPAL not null constraint UNIQUE_PRINCIPAL unique,
    KEYTAB blob constraint NOT_NULL_KEYTAB not null,
    constraint PK_CREDENTIALS primary key (CREDENTIAL_ID)
);

create table HOSTS (
    HOST_ID number constraint NOT_NULL_HOST_ID not null,
    OPTIMISTIC_LOCK_VERSION number constraint NOT_NULL_OPT_LOCK_VER not null,
    HOST_IDENTIFIER varchar2(255) constraint NOT_NULL_HOST_IDENTIFIER not null constraint UNIQUE_HOST_IDENTIFIER unique,
    NAME varchar2(255),
    IP_ADDRESS varchar2(255),
    RACK_ID varchar2(255),
    STATUS varchar2(255),
    constraint PK_HOSTS primary key (HOST_ID)
);

create table PROCESSES (
    PROCESS_ID number constraint NOT_NULL_PROCESS_ID not null,
    NAME varchar2(255) constraint NOT_NULL_PROCESS_NAME not null,
    ROLE_ID number,
    COMMAND_ID number,
    HOST_ID number constraint NOT_NULL_PS_HOST_ID not null,
    PROCESS_USER varchar2(255),
    PROCESS_GROUP varchar2(255),
    PROGRAM varchar2(255),
    ARGUMENTS clob,
    RUNNING number(1,0),
    RUN_GENERATION number,
    CONFIG_GENERATION number constraint NOT_NULL_PS_CONFIG_GEN not null,
    ONE_OFF number(1,0),
    CONFIGURATION_DATA blob,
    STATUS_LINKS clob,
    RESOURCES clob,
    ENVIRONMENT clob,
    REFRESH_FILES clob,
    CREATED_INSTANT number default 0 constraint NOT_NULL_PS_CREATED_INSTANT not null,
    constraint PK_PROCESSES primary key (PROCESS_ID)
);

create table ROLES (
    ROLE_ID number constraint NOT_NULL_ROLE_ID not null,
    NAME varchar2(255) constraint NOT_NULL_ROES_NAME not null constraint UNIQUE_ROLE_NAME unique,
    HOST_ID number,
    ROLE_TYPE varchar2(255),
    CONFIGURED_STATUS varchar2(255),
    STATUS varchar2(255),
    CONFIG_GENERATION number constraint NOT_NULL_ROLES_CONFIG_GEN not null,
    SERVICE_ID number,
    constraint PK_ROLES primary key (ROLE_ID)
);

create table SERVICES (
    SERVICE_ID number constraint NOT_NULL_SERVICE_ID not null,
    OPTIMISTIC_LOCK_VERSION number constraint NOT_NULL_SRV_OPT_LOCK_VER not null,
    NAME varchar2(255) constraint NOT_NULL_SRV_NAME not null constraint UNIQUE_SRV_NAME unique,
    SERVICE_TYPE varchar2(255),
    CONFIG_REVISION_ID number,
    constraint PK_SERVICES primary key (SERVICE_ID)
);

create table SETTINGS (
    SETTINGS_ID number not null,
    OPTIMISTIC_LOCK_VERSION number not null,
    HEARTBEAT_INTERVAL number default 5 not null,
    MISSED_HB_CONCERNING number default 5 not null,
    MISSED_HB_BAD number default 10 not null,
    PASSWORD_HASH varchar2(255),
    PASSWORD_SALT number default 0 not null,
    WEB_TLS number(1,0) default 0 not null,
    AGENT_TLS number(1,0) default 0 not null,
    NEED_AGENT_VALIDAITON number(1,0) default 0 not null,
    KEYSTORE_PATH clob,
    KEYSTORE_PASSWORD varchar2(255),
    TRUSTSTORE_PATH clob,
    TRUSTSTORE_PASSWORD varchar2(255),
    HTTP_PORT number default 7180 not null,
    HTTPS_PORT number default 7183 not null,
    AGENT_PORT number default 7182 not null,
    SECURITY_REALM varchar2(255) default 'HADOOP.COM' not null,
    ALLOW_USAGE_DATA number(1,0) default 0 not null,
    PHONE_HOME number(1,0) default 0 not null,
    LICENSE_KEY clob,
    USING_HELP_FROM_CCP number(1,0) default 0 not null,
    LICENSE_FIRST_APPLIED number(1,0) default 0 not null,
    primary key (SETTINGS_ID)
);

create table USERS (
    USER_ID number constraint NOT_NULL_USER_ID not null,
    USER_NAME varchar2(255) constraint NOT_NULL_USER_NAME not null constraint UNIQUE_USER_NAME unique,
    PASSWORD_HASH varchar2(255) constraint NOT_NULL_PWD_HASH not null,
    PASSWORD_SALT number constraint NOT_NULL_PWD_SALT not null,
    constraint PK_USERS primary key (USER_ID)
);

create table USER_ROLES (
    USER_ROLE_ID number constraint NOT_NULL_USER_ROLE_ID not null,
    USER_ROLE varchar2(255) constraint NOT_NULL_USER_ROLE not null,
    USER_ID number constraint NOT_NULL_UR_USER_ID not null,
    constraint PK_USER_ROLES primary key (USER_ROLE_ID)
);

create index IDX_COMMAND_HOST on COMMANDS (HOST_ID);

create index IDX_COMMAND_PARENT on COMMANDS (PARENT_ID);

create index IDX_COMMAND_ROLE on COMMANDS (ROLE_ID);

create index IDX_COMMAND_SERVICE on COMMANDS (SERVICE_ID);

create index IDX_COMMAND_ACTIVE on COMMANDS (ACTIVE);

alter table COMMANDS
    add constraint FK_COMMAND_HOST
    foreign key (HOST_ID)
    references HOSTS
    deferrable initially immediate;

alter table COMMANDS
    add constraint FK_COMMAND_ROLE
    foreign key (ROLE_ID)
    references ROLES
    deferrable initially immediate;

alter table COMMANDS
    add constraint FK_COMMAND_PARENT
    foreign key (PARENT_ID)
    references COMMANDS
    deferrable initially immediate;

alter table COMMANDS
    add constraint FK_COMMAND_SERVICE
    foreign key (SERVICE_ID)
    references SERVICES
    deferrable initially immediate;

create index IDX_CONFIG_ROLE on CONFIGS (ROLE_ID);

create index IDX_CONFIG_SERVICE on CONFIGS (SERVICE_ID);

create index IDX_CONFIG_CONFIG_REVISION on CONFIGS (REVISION_ID);

alter table CONFIGS
    add constraint FK_CONFIG_CONFIG_REVISION
    foreign key (REVISION_ID)
    references CONFIG_REVISIONS
    deferrable initially immediate;

alter table CONFIGS
    add constraint FK_CONFIG_ROLE
    foreign key (ROLE_ID)
    references ROLES
    deferrable initially immediate;

alter table CONFIGS
    add constraint FK_CONFIG_SERVICE
    foreign key (SERVICE_ID)
    references SERVICES
    deferrable initially immediate;

create index IDX_CONFIG_REVISION_SERVICE on CONFIG_REVISIONS (SERVICE_ID);

alter table CONFIG_REVISIONS
    add constraint FK_CONFIG_REVISION_SERVICE
    foreign key (SERVICE_ID)
    references SERVICES
    deferrable initially immediate;

create index IDX_PROCESS_ROLE on PROCESSES (ROLE_ID);

create index IDX_PROCESS_HOST on PROCESSES (HOST_ID);

create index IDX_PROCESS_COMMAND on PROCESSES (COMMAND_ID);

alter table PROCESSES
    add constraint FK_PROCESS_HOST
    foreign key (HOST_ID)
    references HOSTS
    deferrable initially immediate;

alter table PROCESSES
    add constraint FK_PROCESS_ROLE
    foreign key (ROLE_ID)
    references ROLES
    deferrable initially immediate;

alter table PROCESSES
    add constraint FK_PROCESS_COMMAND
    foreign key (COMMAND_ID)
    references COMMANDS
    deferrable initially immediate;

create index IDX_ROLE_SERVICE on ROLES (SERVICE_ID);

create index IDX_ROLE_HOST on ROLES (HOST_ID);

alter table ROLES
    add constraint FK_ROLE_HOST
    foreign key (HOST_ID)
    references HOSTS
    deferrable initially immediate;

alter table ROLES
    add constraint FK_ROLE_SERVICE
    foreign key (SERVICE_ID)
    references SERVICES
    deferrable initially immediate;

create index IDX_SERVICE_CONFIG_REVISION on SERVICES (CONFIG_REVISION_ID);

alter table SERVICES
    add constraint FK_SERVICE_CONFIG_REVISION
    foreign key (CONFIG_REVISION_ID)
    references CONFIG_REVISIONS
    deferrable initially immediate;

create index IDX_USER_ROLE on USER_ROLES (USER_ID);

alter table USER_ROLES
    add constraint FK_USER_ROLE
    foreign key (USER_ID)
    references USERS
    deferrable initially immediate;
