// Copyright (c) 2012 Cloudera, Inc.  All rights reserved.
define([
	"cloudera/common/I18n",
	"cloudera/common/AutoUpdatedContent"
], function (I18n, AutoUpdatedContent) {

	/**
	 * Encapsulates the install details dialog.
	 * options = {
	 *    detailsDialogSelector: "#someId"
	 * };
	 */
	return function(options) {
		var $dialog = $(options.detailsDialogSelector),
			$modalBody = $dialog.find(".modal-body"),
			$modalHeader = $dialog.find(".modal-header"),
			$modalFooter = $dialog.find(".modal-footer");

		var ignoreAnchorEvent = function(evt) {
			if (evt) {
				evt.preventDefault();
			}
		};

		var hide = function(evt) {
			ignoreAnchorEvent(evt);
			$dialog.modal("hide");
		};

		$dialog.find(".close").click(hide);
		$dialog.find(".closeDetailsBtn").click(hide);

		// Handles the clicking event on the current state/failed state.
		var highlight = function($elem) {
			$elem.effect("highlight", {}, 2000);
		};

		var scrollTo = function($elem) {
			if ($elem && $elem.length > 0) {
				$modalBody.scrollTop(0);
				$modalBody.scrollTop($elem.offset().top - $modalBody.offset().top);
				highlight($elem.next("div"));
			}
		};

		var onCurrentStateClicked = function(evt) {
			ignoreAnchorEvent(evt);
			scrollTo($modalBody.find("h3:last"));
		};

		var onFailedStateClicked = function(evt) {
			ignoreAnchorEvent(evt);
			var $lastSectionWithError = null, $lastSection = null;

			// Determine the last section before rollback.
			$modalBody.find("h3").each(function(i, elem) {
				var $elem = $(elem);
				if ($elem.is(".node-progress-ROLLBACK")) {
					$lastSectionWithError = $lastSection;
				}
				$lastSection = $elem;
			});
			$lastSection = null;

			// Scroll to the last section with an error.
			scrollTo($lastSectionWithError);
		};

		$modalHeader.find(".currentState").click(onCurrentStateClicked);
		$modalHeader.find(".failedState").click(onFailedStateClicked);

		// Auto refreshes the dialog content.
		var detailsContentSelector = options.detailsDialogSelector + " .detailsContent";
		var firstUpdateOnShow = true;
		var autoUpdaterOptions = {
			url: "",
			urlParams: {},
			updateIntervalInMS: 5000,
			checkVisibilityIntervalInMS: 500,
			containerSelector: detailsContentSelector,
			afterUpdate: function() {
				if (firstUpdateOnShow) {
					if ($modalHeader.find(".failedState span").length > 0) {
						$modalHeader.find(".failedState").trigger("click");
					} else if ($modalHeader.find(".currentState span").length > 0) {
						$modalHeader.find(".currentState").trigger("click");
					}
					firstUpdateOnShow = false;
				}
			}
		};
		var detailsAutoUpdater = new AutoUpdatedContent(autoUpdaterOptions);

		// Clear the active class for the pause/play button.
		var clearActiveState = function() {
			$modalFooter.find('.pauseUpdateDetails,.playUpdateDetails').removeClass("active");
		};

		// Turn off auto refresh.
		var onPauseUpdateDetailsClicked = function(evt) {
			ignoreAnchorEvent(evt);
			var $target = $(evt.currentTarget);

			clearActiveState();
			$target.addClass("active");
			detailsAutoUpdater.stop();
			$modalFooter.find(".pauseUpdateDetailsReason").removeClass("hidden");
		};

		// Turn on auto refresh.
		var onPlayUpdateDetailsClicked = function(evt) {
			ignoreAnchorEvent(evt);
			var $target = $(evt.currentTarget);

			clearActiveState();
			$target.addClass("active");
			detailsAutoUpdater.start();
			$modalFooter.find(".pauseUpdateDetailsReason").addClass("hidden");
		};

		$modalFooter.find('.pauseUpdateDetails').click(onPauseUpdateDetailsClicked);
		$modalFooter.find('.playUpdateDetails').click(onPlayUpdateDetailsClicked);


		// Clicking on a Details link requires updating the URL
		// and invoke content update immediately.
		var show = function(url) {
			// restore the Install Details dialog "Loading..." placeholder text
			$(detailsContentSelector).html(I18n.t("ui.loading"));

			// Restore the header text to the initial state.
			$modalHeader.find(".failedState").html("");
			$modalHeader.find(".currentState").html("");

			firstUpdateOnShow = true;

			detailsAutoUpdater.url = url;
			// This triggers auto update.
			$modalFooter.find('.playUpdateDetails').trigger("click");
			$dialog.modal('show');
		};

		// Expose a single method: show
		return {
			show: show
		};
	};
});
